const { assert, expect } = require('chai');
const sinon = require('sinon');
const fs = require('fs');
const awsProfileHandler = require('aws-profile-handler');
const mkdirp = require('mkdirp');
const awsSetupHelper = require('@src/commands/init/aws-setup-helper');
const ui = require('@src/commands/init/ui');
const profileHelper = require('@src/utils/profile-helper');
const messages = require('@src/commands/init/messages');
const stringUtils = require('@src/utils/string-utils');
const CONSTANTS = require('@src/utils/constants');

describe('Command: Init - AWS setup helper test', () => {
    describe('# resolve aws profile', () => {
        let sandbox;
        beforeEach(() => {
            sandbox = sinon.createSandbox();
            sandbox.stub(ui, 'confirmSettingAws');
            sandbox.stub(fs, 'existsSync');
            sandbox.stub(awsProfileHandler, 'listProfiles');
            sandbox.stub(stringUtils, 'isNonBlankString');
            sandbox.stub(ui, 'createNewOrSelectAWSProfile');
            sandbox.stub(profileHelper, 'setupProfile');
            sandbox.stub(awsProfileHandler, 'getProfileCredentials');
            sandbox.stub(ui, 'selectEnvironmentVariables');
            sandbox.stub(console, 'log');
            sandbox.stub(ui, 'requestAwsProfileName');
            sandbox.stub(ui, 'addNewCredentials');
            sandbox.stub(awsProfileHandler, 'addProfile');
        });

        it('| check for error while resolving aws profile', () => {
            // setup
            const ERROR = 'error while resolving profile';
            ui.confirmSettingAws.withArgs(sinon.match.func).throws(ERROR);

            // call
            awsSetupHelper.resolve('askProfile', (error) => {
                // verify
                expect(error.name).equal(ERROR);
            });
        });

        it('| confirm profile creation: no existing profiles', () => {
            // setup
            const AWS_PROFILE = 'awsProfile';
            fs.existsSync.withArgs(sinon.match.string).returns(false);
            stringUtils.isNonBlankString.withArgs(sinon.match.any).returns(false);
            awsProfileHandler.listProfiles.returns([AWS_PROFILE]);
            ui.confirmSettingAws.callsArgOnWith(0, {}, true);
            ui.createNewOrSelectAWSProfile.callsArgOnWith(1, {}, AWS_PROFILE);
            ui.requestAwsProfileName.callsArgOnWith(0, {}, AWS_PROFILE);
            awsProfileHandler.getProfileCredentials.withArgs(sinon.match.string).returns(null);
            ui.addNewCredentials.callsArgOnWith(0, {}, 'credentials');

            // call
            awsSetupHelper.resolve('askProfile', (error, response) => {
                // verify
                expect(error).to.be.null;
                expect(response).equal(AWS_PROFILE);
                expect(profileHelper.setupProfile.called).equal(true);
                expect(console.log.args[0][0])
                    .equal(`AWS profile "${response}" was successfully created. The details are recorded in aws credentials file ($HOME/.aws/credentials).`);
            });
        });

        it('| user confirms setting up aws profile, with no environment variables, listProfiles throws error', () => {
            // setup
            const AWS_PROFILE = 'awsProfile';
            const ERROR = 'no previous profiles found';
            fs.existsSync.withArgs(sinon.match.string).returns(false);
            stringUtils.isNonBlankString.withArgs(sinon.match.any).returns(false);
            awsProfileHandler.listProfiles.throws('message', ERROR);
            ui.confirmSettingAws.callsArgOnWith(0, {}, true);
            ui.createNewOrSelectAWSProfile.callsArgOnWith(1, {}, AWS_PROFILE);

            // call
            awsSetupHelper.resolve('askProfile', (error) => {
                // verify
                expect(error).equal(ERROR);
            });
        });

        it('| create new aws profile when user selects "create new profile"', () => {
            // setup
            const AWS_PROFILE = 'Create new profile';
            const ERROR = 'no previous profiles found';
            fs.existsSync.withArgs(sinon.match.string).returns(true);
            stringUtils.isNonBlankString.withArgs(sinon.match.any).returns(false);
            awsProfileHandler.listProfiles.returns([AWS_PROFILE]);
            ui.confirmSettingAws.callsArgOnWith(0, {}, true);
            ui.createNewOrSelectAWSProfile.callsArgOnWith(1, {}, AWS_PROFILE);
            ui.requestAwsProfileName.callsArgOnWith(0, {}, 'awsProfile');
            awsProfileHandler.getProfileCredentials.returns(null);
            ui.addNewCredentials.callsArgOnWith(0, {}, {});

            // call
            awsSetupHelper.resolve('askProfile', (error, response) => {
                // verify
                expect(error).to.be.null;
                expect(response).equal('awsProfile');
                expect(profileHelper.setupProfile.called).equal(true);
                expect(console.log.args[0][0])
                    .equal(`AWS profile "${response}" was successfully created. The details are recorded in aws credentials file ($HOME/.aws/credentials).`);
            });
        });

        it('| user confirms setting up aws profile, with valid environment variables and selects yes', () => {
            // setup
            fs.existsSync.withArgs(sinon.match.string).returns(false);
            stringUtils.isNonBlankString.withArgs(sinon.match.any).returns(true);
            awsProfileHandler.listProfiles.withArgs(sinon.match.string).returns([]);
            ui.confirmSettingAws.callsArgOnWith(0, {}, true);
            ui.selectEnvironmentVariables.callsArgOnWith(0, {}, 'Yes');

            // call
            awsSetupHelper.resolve('askProfile', (error, response) => {
                // verify
                expect(error).to.be.null;
                expect(response).equal(CONSTANTS.PLACEHOLDER.ENVIRONMENT_VAR.AWS_CREDENTIALS);
            });
        });

        it('| user confirms setting up aws profile, with valid environment variables and selects no, with existing AWS profiles', () => {
            // setup
            fs.existsSync.withArgs(sinon.match.string).returns(true);
            stringUtils.isNonBlankString.withArgs(sinon.match.any).returns(true);
            awsProfileHandler.listProfiles.withArgs(sinon.match.string).returns(['awsProfile']);
            awsProfileHandler.listProfiles.returns([]);
            ui.confirmSettingAws.callsArgOnWith(0, {}, true);
            ui.selectEnvironmentVariables.callsArgOnWith(0, {}, 'No');
            ui.createNewOrSelectAWSProfile.callsArgOnWith(1, {}, 'awsProfile');

            // call
            awsSetupHelper.resolve('askProfile', (error, response) => {
                // verify
                expect(error).to.be.null;
                expect(response).equal('awsProfile');
            });
        });

        it('| user rejects setting up aws profile', () => {
            // setup
            fs.existsSync.withArgs(sinon.match.string).returns(false);
            awsProfileHandler.listProfiles.withArgs(sinon.match.string).returns([]);
            ui.confirmSettingAws.callsArgOnWith(0, {}, false);

            // call
            awsSetupHelper.resolve('askProfile', () => {
                // verify
                expect(console.log.args[0][0]).equal(messages.SKIP_AWS_INITIALIZATION);
            });
        });

        afterEach(() => {
            sandbox.restore();
        });
    });

    describe('# setup aws profile', () => {
        let sandbox;
        beforeEach(() => {
            sandbox = sinon.createSandbox();
            sandbox.stub(fs, 'existsSync');
            sandbox.spy(mkdirp, 'sync');
            sandbox.spy(fs, 'closeSync');
            sandbox.spy(fs, 'openSync');
            sandbox.stub(awsProfileHandler, 'listProfiles');
            sandbox.stub(ui, 'addNewCredentials');
            sandbox.stub(awsProfileHandler, 'addProfile');
            sandbox.stub(console, 'log');
            sandbox.stub(ui, 'confirmOverwritingProfile');
            sandbox.stub(awsProfileHandler, 'getProfileCredentials');
            sandbox.stub(ui, 'requestAwsProfileName');
            sandbox.stub(profileHelper, 'setupProfile');
        });

        it('| AWS credentials do not exist ', () => {
            // setup
            fs.existsSync.withArgs(sinon.match.string).returns(false);
            ui.addNewCredentials.callsArgOnWith(0, {}, {});
            awsProfileHandler.listProfiles.returns([]);

            // call
            awsSetupHelper.setup((error) => {
                // verify
                expect(mkdirp.sync.withArgs(sinon.match.string).called).equal(true);
                expect(fs.closeSync.withArgs(sinon.match.number).called).equal(true);
                expect(fs.openSync.withArgs(sinon.match.string, sinon.match.string).called).equal(true);
                expect(error).to.be.null;
            });
        });

        it('| AWS credentials exist. user confirms overwriting profile ', () => {
            // setup
            fs.existsSync.withArgs(sinon.match.string).returns(false);
            ui.addNewCredentials.callsArgOnWith(0, {}, {});
            ui.confirmOverwritingProfile.callsArgOnWith(1, {}, true);
            awsProfileHandler.listProfiles.returns([]);
            awsProfileHandler.getProfileCredentials.returns('credentials');

            // call
            awsSetupHelper.setup((error, response) => {
                // verify
                expect(error).to.be.null;
                expect(response).equal(CONSTANTS.COMMAND.INIT.AWS_DEFAULT_PROFILE_NAME);
            });
        });

        it('| AWS credentials exist. user denies overwriting profile ', () => {
            // setup
            fs.existsSync.withArgs(sinon.match.string).returns(false);
            ui.addNewCredentials.callsArgOnWith(0, {}, {});
            ui.confirmOverwritingProfile.callsArgOnWith(1, {}, false);
            awsProfileHandler.listProfiles.returns([]);
            awsProfileHandler.getProfileCredentials.withArgs(sinon.match.string)
                .onFirstCall().returns('credentials')
                .onSecondCall()
                .returns(null);
            ui.addNewCredentials.callsArgOnWith(0, {}, {});

            // call
            awsSetupHelper.setup((error, response) => {
                // verify
                expect(error).to.be.null;
                expect(response).equal(CONSTANTS.COMMAND.INIT.AWS_DEFAULT_PROFILE_NAME);
            });
        });

        it('| setup credentials: profile name is null ', () => {
            // setup
            const AWS_PROFILE = 'awsProfile';
            const ASK_PROFILE = 'default';
            fs.existsSync.withArgs(sinon.match.string).returns(false);
            ui.confirmOverwritingProfile.callsArgOnWith(1, {}, true);
            awsProfileHandler.listProfiles.returns([ASK_PROFILE]);
            awsProfileHandler.getProfileCredentials.returns('credentials');
            ui.requestAwsProfileName.callsArgOnWith(0, {}, AWS_PROFILE);

            // call
            awsSetupHelper.setup((error, response) => {
                // verify
                expect(error).to.be.null;
                expect(response).equal('awsProfile');
            });
        });

        afterEach(() => {
            sandbox.restore();
        });
    });
});
